<?php
/**
 * Cette classe fournit une interface pour les notifications
 * @copyright   &copy; 2005-2019 PHPBoost
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL-3.0
 * @author      Kévin BRISSEZ <brissez.kevin@gmail.com>
 * @version     PHPBoost 5.2 - last update: 2019 11 01
 * @since       PHPBoost 5.0 - 2019 07 01
 */

class NotificationsHomeController extends ModuleController
  {
    private $view;
    private $user;
    
    public function execute(HTTPRequestCustom $request)
      {
        
        $this->init();
        $this->build_view($request);
        
        $user = AppContext::get_current_user();
        
        if (!$user->check_level(User::MEMBER_LEVEL))
          {
            $error_controller = PHPBoostErrors::user_not_authorized();
            DispatchManager::redirect($error_controller);
          }
        
        return $this->generate_response($request);
      }
    
    public function init()
      {
        $this->lang = LangLoader::get('common', 'notifications');
        $this->tpl  = new FileTemplate('notifications/notifications.tpl');
        $this->tpl->add_lang($this->lang);
        $this->config = NotificationsConfig::load();
      }
    
    public function build_view(HTTPRequestCustom $request)
      {
        $user              = AppContext::get_current_user();
        // On compte les notifications de l'utilisateur quand il est le destinataire 
        $nbr_notifications = NotificationsService::count('WHERE recipient_id=:recip_id AND reading=0', array(
            'recip_id' => $user->get_id()
        ));
        
        // Variable pour la pagination qui fait appel à la fonction get_pagination, plus bas
        $page       = AppContext::get_request()->get_getint('page', 1);
        $pagination = $this->get_pagination($nbr_notifications, $page);
        
        // S'il y a au moins une notification on en informe l'utilisateur.
        if ($nbr_notifications > 0)
          {
            
            $list_notifications = PersistenceContext::get_querier()->select('SELECT n.id AS id_notification, n.sender_id, n.recipient_id, n.content, n.date, n.reading, s.display_name AS username_sender, r.display_name AS username_recipient, a.user_avatar AS sender_avatar FROM ' . PREFIX . 'notifications n LEFT JOIN ' . PREFIX . 'member s ON s.user_id = n.sender_id LEFT JOIN ' . PREFIX . 'member r ON r.user_id = n.recipient_id LEFT JOIN ' . PREFIX . 'member_extended_fields a ON a.user_id = s.user_id WHERE n.recipient_id=:recip_id AND n.reading = 0 ORDER BY n.id DESC LIMIT :number_items_per_page OFFSET :display_from', array(
                'recip_id' => $user->get_id(),
                'number_items_per_page' => $pagination->get_number_items_per_page(),
                'display_from' => $pagination->get_display_from()
            ), SelectQueryResult::FETCH_ASSOC);
            
            
            // On fait une boucle qui liste toutes les notifications de l'utilisateur quand Lu = 0
            while ($notifications = $list_notifications->fetch())
              {
                // Vérification de l'avatar, s'il est inexistant on le remplace par celui par défaut
                $user_accounts_config = UserAccountsConfig::load();
                if (empty($notifications['sender_avatar']))
                  {
                    $avatar_sender = ($user_accounts_config->is_default_avatar_enabled() == '1') ? '<img src="' . PATH_TO_ROOT . '/templates/' . AppContext::get_current_user()->get_theme() . '/images/' . $user_accounts_config->get_default_avatar_name() . '" alt="' . LangLoader::get_message('avatar', 'user-common') . '" />' : '';
                  }
                else
                  {
                    $avatar_sender = '<img src="' . Url::to_rel($notifications['sender_avatar']) . '" alt="' . LangLoader::get_message('avatar', 'user-common') . '"/>';
                  }
                
                $this->tpl->assign_block_vars('list_notifications', array(
                    'ID' => $notifications['id_notification'],
                    'CONTENT' => $notifications['content'],
                    'SENDER_AVATAR' => $avatar_sender,
                    'DATE' => NotificationsFormatDate::getFormatDate('@' . $notifications['date']),
                    'READ' => $notifications['reading'] == 1 ? false : true,
                    'U_DELETE' => NotificationsUrlBuilder::delete($notifications['id_notification'])->rel(),
                    'U_ARCHIVE_ONE' => $this->config->get_archivage_auto() == true ? '' : PATH_TO_ROOT . "/notifications/?read_one=" . $notifications['id_notification'] . "&token=" . AppContext::get_session()->get_token(),
                    'L_ARCHIVED_THIS' => $this->lang['notification.archive.this.notification']
                ));
                
              }
            
          }
        
        // On compte le nombre de notification non lu par le destinataire
        $nbr_notifications_non_lu = NotificationsService::count('WHERE recipient_id=:recip_id AND reading = 0', array(
            'recip_id' => $user->get_id()
        ));
        $this->tpl->put_all(array(
            'C_NO_NOTIFICATION' => $nbr_notifications_non_lu == 0,
            'C_PAGINATION' => $nbr_notifications > NotificationsConfig::load()->get_items_per_page(),
            'PAGINATION' => $pagination->display(),
            'U_MARK_AS_READ' => PATH_TO_ROOT . "/notifications/?read=1&token=" . AppContext::get_session()->get_token(),
            'U_ARCHIVE' => PATH_TO_ROOT . "/notifications/archives",
            'L_MARK_AS_READ' => $this->lang['notifications.mark.as.read'],
            'L_ARCHIVE' => $this->lang['notifications.my.archives'],
            
            // DISPLAY_DEL_MARK  "marquer comme vu" si l'admin a désactiver la suppression automatique
            'DISPLAY_DEL_MARK' => $this->config->get_del_notif_after_view() == true ? false : true
        ));
        
        // Paramètres READ dans l'URL
        $mark_notifications_as_read = $request->get_getint('read', 0);
        // Si le paramètre read = 1 et si l'admin a désactiver les suppression automatique on peut lire toutes les notifications du membre et donc ajouter reading = 1 dans la bdd
        if (isset($mark_notifications_as_read) AND $mark_notifications_as_read == 1)
          {
            AppContext::get_session()->csrf_get_protect();
            
            if ($this->config->get_del_notif_after_view() == FALSE AND $nbr_notifications_non_lu >= 1)
              {
                $recipient_id = $user->get_id();
                NotificationsService::mark_notifications_as_read($recipient_id);
                AppContext::get_response()->redirect(($request->get_url_referrer() ? $request->get_url_referrer() : NotificationsUrlBuilder::home()), LangLoader::get_message('notifications.mark.all.success', 'common', 'notifications'));
              }
          }
        
        // Suppression des notifications dès la lecture puisque l'admin à activer la suppression automatique des notification (On exécute cette requête seulement si y'a au moins une notification, sinon ça sert à rien ...)
        if ($nbr_notifications >= 1 AND $this->config->get_del_notif_after_view() == TRUE)
          {
            NotificationsService::delete('WHERE recipient_id=:id', array(
                'id' => $user->get_id()
            ));
          }
        
        // Si l'archivage automatique est activé alors on lance l'archivage automatique mais pour cela il faut qu'il y ai au moins une notification non lu ..
        elseif ($this->config->get_archivage_auto() == TRUE AND $nbr_notifications_non_lu >= 1)
          {
            $recipient_id = $user->get_id();
            NotificationsService::mark_notifications_as_read($recipient_id);
          }
        
        
        // Si une seule notification a été lue
        $mark_one_notification_as_read = $request->get_getint('read_one', 0);
        if (isset($mark_one_notification_as_read) AND !empty($mark_one_notification_as_read))
          {
            // On demande un token de sécurité pour éviter qu'on nous envoi une url qui permet de faire une action qu'on veut pas ...
            AppContext::get_session()->csrf_get_protect();
            // On vérifie déjà que l'archivage automatique est désactivée ainsi que la suppression automatique
            if ($this->config->get_archivage_auto() == FALSE AND $this->config->get_del_notif_after_view() == FALSE)
              {
                $getNotification = $this->get_notification($mark_one_notification_as_read);
                
                // Si le membre est "propriétaire" de la notification alors il peut l'archiver
                if ($getNotification['recipient_id'] == $user->get_id())
                  {
                    NotificationsService::mark_one_notification_as_read($mark_one_notification_as_read, $getNotification['recipient_id']);
                    AppContext::get_response()->redirect(($request->get_url_referrer() ? $request->get_url_referrer() : NotificationsUrlBuilder::home()), LangLoader::get_message('notification.archived.success', 'common', 'notifications'));
                  }
                // Sinon s'il n'est pas propriétaire ou si la notification n'existe pas on ne fait rien et redirection sur la page des notifications
                else
                  {
                    AppContext::get_response()->redirect(NotificationsUrlBuilder::home());
                  }
              }
          }
        
        return $this->view;
        
      }
    
    private function get_notification(int $mark_one_notification_as_read)
      {
        # $mark_one_notification_as_read = id de la notification dans l'url
        try
          {
            $notification = PersistenceContext::get_querier()->select_single_row(PREFIX . 'notifications', array(
                'id, recipient_id'
            ), 'WHERE id=:id_notification', array(
                'id_notification' => $mark_one_notification_as_read
            ));
          }
        catch (RowNotFoundException $e)
          {
            return false;
          }
        
        return $notification;
      }
    
    private function get_pagination($nbr_notifications, $page)
      {
        $pagination = new ModulePagination($page, $nbr_notifications, (int) NotificationsConfig::load()->get_items_per_page());
        $pagination->set_url(NotificationsUrlBuilder::home('%d'));
        
        if ($pagination->current_page_is_empty() && $page > 1)
          {
            AppContext::get_response()->redirect(NotificationsUrlBuilder::home());
          }
        
        return $pagination;
      }
    private function generate_response(HTTPRequestCustom $request)
      {
        $response = new SiteDisplayResponse($this->tpl);
        $response->get_graphical_environment()->set_page_title($this->lang['module_title']);
        return $response;
      }
  }